@@ -4,10 +4,14 @@
 getOption <- appendVarArgs(getOption)
 inherits <- appendVarArgs(inherits)
 parse <- appendVarArgs(parse)
+if (exists("timestamp", mode="function"))
+  timestamp <- appendVarArgs(timestamp)
 
 
 ############################################################################
 # HISTORY:
+# 2006-03-28
+# o Added fix to fresh timestamp() in utils.
 # 2005-05-26
 # o Added fix for default getOption().
 # 2005-02-15
